@@ -144,7 +144,8 @@ lmCombine <- function(data, ic=c("AICc","AIC","BIC","BICc"), bruteforce=FALSE, s
     }
 
     # The gsub is needed in order to remove accidental special characters
-    colnames(data) <- gsub("\`","",colnames(data),ignore.case=TRUE);
+    # colnames(data) <- gsub("\`","",colnames(data),ignore.case=TRUE);
+    colnames(data) <- make.names(colnames(data), unique=TRUE);
 
     # Define cases, when to use ALM
     distribution <- distribution[1];
