@@ -5,18 +5,20 @@
 #' This function produces rolling origin forecasts using the \code{data} and a
 #' \code{call} passed as parameters. The function can do all of that either in
 #' serial or in parallel, but it needs \code{foreach} and either \code{doMC}
-#' (Linux only), \code{doParallel} or \code{doSNOW} packages installed in order
-#' to do the latter.
+#' (Linux only) or \code{doParallel} packages installed in order to do the latter.
 #'
 #' This is a dangerous function, so be careful with the call that you pass to
-#' it, and make sure that it is well formulated before the execution.
+#' it, and make sure that it is well formulated before the execution. Also, do not
+#' forget to provide the value that needs to be returned or you might end up with
+#' very messy results.
 #'
 #' For more details and more examples of usage, please see vignette for the function.
 #' In order to do that, just run the command: \code{vignette("ro","greybox")}
 #'
 #' @keywords ts
 #'
-#' @param data Data vector or ts object passed to the function.
+#' @param data Data vector or ts object with the response variable passed to the
+#' function.
 #' @param h The forecasting horizon.
 #' @param origins The number of rolling origins.
 #' @param call The call that is passed to the function. The call must be in quotes.
@@ -68,12 +70,14 @@
 #'
 #' @examples
 #'
-#' x <- rnorm(100,0,1)
+#' y <- rnorm(100,0,1)
 #' ourCall <- "predict(arima(x=data,order=c(0,1,1)),n.ahead=h)"
+#' # NOTE that the "data" needs to be used in the call, not "y".
+#' # This way we tell the function, where "y" should be used in the call of the function.
 #'
 #' # The default call and values
 #' ourValue <- "pred"
-#' ourRO <- ro(x, h=5, origins=5, ourCall, ourValue)
+#' ourRO <- ro(y, h=5, origins=5, ourCall, ourValue)
 #'
 #' # We can now plot the results of this evaluation:
 #' plot(ourRO)
@@ -81,28 +85,28 @@
 #' # You can also use dolar sign
 #' ourValue <- "$pred"
 #' # And you can have constant in-sample size
-#' ro(x, h=5, origins=5, ourCall, ourValue, ci=TRUE)
+#' ro(y, h=5, origins=5, ourCall, ourValue, ci=TRUE)
 #'
 #' # You can ask for several values
 #' ourValue <- c("pred","se")
 #' # And you can have constant holdout size
-#' ro(x, h=5, origins=20, ourCall, ourValue, ci=TRUE, co=TRUE)
+#' ro(y, h=5, origins=20, ourCall, ourValue, ci=TRUE, co=TRUE)
 #'
 #' #### The following code will give exactly the same result as above,
 #' #### but computed in parallel using all but 1 core of CPU:
-#' \dontrun{ro(x, h=5, origins=20, ourCall, ourValue, ci=TRUE, co=TRUE, parallel=TRUE)}
+#' \dontrun{ro(y, h=5, origins=20, ourCall, ourValue, ci=TRUE, co=TRUE, parallel=TRUE)}
 #'
 #' #### If you want to use functions from forecast package, please note that you need to
 #' #### set the values that need to be returned explicitly. There are two options for this.
 #' # Example 1:
 #' \dontrun{ourCall <- "forecast(ets(data), h=h, level=95)"
 #' ourValue <- c("mean", "lower", "upper")
-#' ro(x,h=5,origins=5,ourCall,ourValue)}
+#' ro(y,h=5,origins=5,ourCall,ourValue)}
 #'
 #' # Example 2:
 #' \dontrun{ourCall <- "forecast(ets(data), h=h, level=c(80,95))"
 #' ourValue <- c("mean", "lower[,1]", "upper[,1]", "lower[,2]", "upper[,2]")
-#' ro(x,h=5,origins=5,ourCall,ourValue)}
+#' ro(y,h=5,origins=5,ourCall,ourValue)}
 #'
 #' #### A more complicated example using the for loop and
 #' #### several time series
@@ -121,8 +125,8 @@
 #'
 #' ## Start the loop. The important thing here is to use the same variable 'i' as in ourCall.
 #' for(i in 1:3){
-#'     ourdata <- x[,i]
-#'     ourForecasts[,,i] <- ro(data=ourdata,h=6,origins=4,call=ourCall,
+#'     ourData <- x[,i]
+#'     ourForecasts[,,i] <- ro(data=ourData,h=6,origins=4,call=ourCall,
 #'                             value=ourValue,co=TRUE,silent=TRUE)$pred
 #' }
 #'
@@ -131,28 +135,36 @@
 #'
 #' ##### An example with exogenous variables
 #' x <- rnorm(100,0,1)
-#' xreg <- rnorm(100,0,1)
+#' xreg <- matrix(rnorm(200,0,1),100,2,dimnames=list(NULL,c("x1","x2")))
 #'
 #' ## 'counti' is used to define in-sample size of xreg,
 #' ## 'counto' - the size of the holdout sample of xreg
 #'
-#' ourCall <- "predict(arima(x=data, order=c(0,1,1), xreg=xreg[counti]),
-#'             n.ahead=h, newxreg=xreg[counto])"
+#' ourCall <- "predict(arima(x=data, order=c(0,1,1), xreg=xreg[counti,,drop=FALSE]),
+#'             n.ahead=h, newxreg=xreg[counto,,drop=FALSE])"
 #' ourValue <- "pred"
 #' ro(x,h=5,origins=5,ourCall,ourValue)
 #'
+#' ##### Poisson regression with alm
+#' x <- rpois(100,2)
+#' xreg <- cbind(x,matrix(rnorm(200,0,1),100,2,dimnames=list(NULL,c("x1","x2"))))
+#' ourCall <- "predict(alm(x~., data=xreg[counti,,drop=FALSE], distribution='dpois'),
+#'                     newdata=xreg[counto,,drop=FALSE])"
+#' ourValue <- "mean"
+#' testRO <- ro(xreg[,1],h=5,origins=5,ourCall,ourValue,co=TRUE)
+#' plot(testRO)
 #'
 #' ## 'countf' is used to take xreg of the size corresponding to the whole
 #' ## sample on each iteration
 #' ## This is useful when working with functions from smooth package.
 #' ## The following call will return the forecasts from es() function of smooth.
-#' \dontrun{ourCall <- "es(data=data, h=h, xreg=xreg[countf])"
-#' ourValue <- "forecast"
-#' ro(x,h=5,origins=5,ourCall,ourValue)}
+#' \dontrun{ourCall <- "es(data=data, h=h, xreg=xreg[countf,,drop=FALSE])"
+#' \dontrun{ourValue <- "forecast"}
+#' \dontrun{ro(x,h=5,origins=5,ourCall,ourValue)}}
 #'
 #' @export ro
 ro <- function(data,h=10,origins=10,call,value=NULL,
-               ci=FALSE,co=FALSE,silent=TRUE,parallel=FALSE, ...){
+               ci=FALSE,co=TRUE,silent=TRUE,parallel=FALSE, ...){
     # Function makes Rolling Origin for the data using the call
     #    Copyright (C) 2016  Yves Sagaert & Ivan Svetunkov
 
@@ -173,8 +185,8 @@ ro <- function(data,h=10,origins=10,call,value=NULL,
     }
 
     # Check if the data is vector or ts
-    if(!is.numeric(data) & !is.ts(data)){
-        stop("The provided data is not a vector or ts object! Can't work with it!", call.=FALSE);
+    if(!is.numeric(data) && !is.ts(data) && !is.data.frame(data)){
+        stop("The provided data is not a vector or data.frame object! Can't work with it!", call.=FALSE);
     }
 
     # If they asked for parallel, make checks and try to do that
